unit sp_Lib_Delphi;
interface
(*
  -   :
     "Delphi  1:.  "
*)

Uses
  //__IO,

  Dialogs,
  Consts,
  ComObj,
  FileCtrl,
  registry,
  ActiveX,
  ShlObj,

  classes,
  windows,
  SysUtils;



Type
  TMyHandleStream = class(TStream)
  private
    FHandle: Integer;
  protected
    procedure SetSize(NewSize: Longint); override;
  public
    constructor Create(AHandle: Integer);
    function Read(var Buffer; Count: Longint): Longint; override;
    function Write(const Buffer; Count: Longint): Longint; override;
    function Seek(Offset: Longint; Origin: Word): Longint; override;
    property Handle: Integer read FHandle;
  end;

  TMyFileStream = class(TMyHandleStream)
  public
    constructor Create(const FileName: string; Mode: Word);
    destructor Destroy; override;
  end;

  TMyMemoryStream = class(TMemoryStream)
  private
  protected
  public
    destructor Destroy; override;
    procedure MyLoadFromFile(const FileName: string);
  end;



//    .    Windows.
function Get_Path_To_SpecialFolder(CSIDL : cardinal) : string;


//,     Variant
function YesLinkToVariant(HandleAny : Variant) : boolean;


//  List.LoadFromFile
function LoadFromFileToStrings(fn : String;List : TStrings) : integer;

//   CDF-  
function sp_Convert_SDF_to_List(Sx : string; List : TStrings; UnChars : string) : integer;
//    (  )
function GetValueFromList(NumParam: integer; List : TSTrings) : string;


// Handle      (NameWin)
function sp_GetHandleThisWindow(NameWin : string;Nr : byte;YesPartEQ,YesUpCase : Boolean) : hWND;
//      Windows
function sp_GetListOfWindowsNames(List : TStrings) : integer;


//===================================================
//    

//      ,    UnChar
function sp_LTrimUnChar(UnChar , Sx : string) : string;
//      ,    UnChar
function sp_RTrimUnChar(UnChar , Sx : string) : string;
//        ,    UnChar
function sp_AllTrimUnChar(UnChar , Sx : string) : string;
//  
function sp_InversionString(Sx : string) : string;


//,  -   .
function sp_GetCountWordsInString(UnChar, InString : string) : integer;
//,       
function sp_GetWordFromString(UnChar , InString : string; NumWord : integer) : string;

//    
function sp_StrToInt(Sx : string; Def : integer) : integer;

//  
function sp_CompareTwoString(Nr : byte;YesPartEQ,YesUpCase : Boolean;SFind,Sx : String) : Boolean;
//   
function sp_FindInTStrings(Nr : byte;YesPartEQ,YesUpCase : Boolean;SFind : string; List: TStrings) : integer;

//  Value   UTF-8   ANSI
function UTF8ToStrSmart(Value: String): String;


//  List   UTF-8   ANSI
function UTF8_to_Str_List(List : TStrings): integer;


//===================================================


implementation


//*****************************************************************
//*****************************************************************
//*****************************************************************
constructor TMyHandleStream.Create(AHandle: Integer);
begin
  FHandle := AHandle;
end;

function TMyHandleStream.Read(var Buffer; Count: Longint): Longint;
begin
  Result := FileRead(FHandle, Buffer, Count);
  if Result = -1 then Result := 0;
end;

function TMyHandleStream.Write(const Buffer; Count: Longint): Longint;
begin
  Result := FileWrite(FHandle, Buffer, Count);
  if Result = -1 then Result := 0;
end;

function TMyHandleStream.Seek(Offset: Longint; Origin: Word): Longint;
begin
  Result := FileSeek(FHandle, Offset, Origin);
end;

procedure TMyHandleStream.SetSize(NewSize: Longint);
begin
  Seek(NewSize, soFromBeginning);
  Win32Check(SetEndOfFile(FHandle));
end;


function FileMyOpen(const FileName: string): Integer;
begin
  Result := Integer(Windows.CreateFile(PChar(FileName),
                               GENERIC_READ or GENERIC_WRITE,
                               FILE_SHARE_READ OR FILE_SHARE_WRITE,
                               {nil}0,
                               OPEN_EXISTING,
                               FILE_ATTRIBUTE_NORMAL,
                               0));
end;

constructor TMyFileStream.Create(const FileName: string; Mode: Word);
begin
  if Mode = fmCreate then
  begin
    FHandle := FileCreate(FileName);
    if FHandle < 0 then
      raise EFCreateError.CreateResFmt(@SFCreateError, [FileName]);
  end else
  begin
    FHandle := FileMyOpen(FileName);
    if FHandle < 0 then
      raise EFOpenError.CreateResFmt(@SFOpenError, [FileName]);
  end;
end;

destructor TMyFileStream.Destroy;
begin
  if FHandle >= 0 then FileClose(FHandle);
end;


procedure TMyMemoryStream.MyLoadFromFile(const FileName: string);
var
  Stream: TStream;
begin
  //ShowMessage('!  '+FileName);
  Stream := TMyFileStream.Create(FileName, fmOpenRead or fmShareDenyWrite);
  try
    LoadFromStream(Stream);
  finally
    Stream.Free;
  end;
end;


destructor TMyMemoryStream.Destroy;
begin
  inherited Destroy;
end;

//*****************************************************************
//*****************************************************************
//*****************************************************************



//*******************************************************************
function Get_Path_To_SpecialFolder(CSIDL : cardinal) : string;
//    . 
//  Windows.
//  http://www.swissdelphicenter.ch/en/showcode.php?id=233
var
  Allocator: IMalloc;
  SpecialDir: PItemIdList;
  FBuf: array[0..MAX_PATH] of Char;
  PerDir: string;
begin
  Result:='';
  if SHGetMalloc(Allocator) = NOERROR then begin
    SHGetSpecialFolderLocation({Handle}0, CSIDL_APPDATA, SpecialDir);
    SHGetPathFromIDList(SpecialDir, @FBuf[0]);
    Allocator.Free(SpecialDir);
    Result:=string(FBuf);
  end;
end;
//*******************************************************************



//***************************************************************
function YesLinkToVariant(HandleAny : Variant) : boolean;
//,     Variant
begin
 Result:=FALSE;
 if not VarIsEmpty(HandleAny) then Result:=TRUE;
end;
//***************************************************************




function LoadFromFileToStrings(fn : String;List : TStrings) : integer;
//  List.LoadFromFile
//  
Var
  TempStream : TMyMemoryStream;
begin
  Result:=0;
  if List<>NIL then begin
     List.Clear;
     //ShowMessage(fn);
     if FileExists(fn) then begin
        TempStream := TMyMemoryStream.Create;
        TRY
          TempStream.MyLoadFromFile(fn);
          TempStream.Position := 0;
          List.LoadFromStream(TempStream);


          //ShowMessage(IntToStr(List.Count));

        FINALLY
          TempStream.Free;
        END;
     end;
     Result:=List.Count;
  end;
end;



function sp_Convert_SDF_to_List(Sx : string; List : TStrings; UnChars : string) : integer;
//   CDF-  
// Sx  :
//  "12","","","  ","0","5","2","2","2","64"
//     List.
// ""  Sx -    List
//UnChars - ,      Sx
// 
begin
  Result:=0;
  if List<>NIL then begin
     List.Clear;
     //Sx:=sp_AllTrimUnChar('{}, ',Sx);
     Sx:=sp_AllTrimUnChar(UnChars,Sx);
     if length(Sx)>0 then begin
        List.CommaText:=Sx;
     end;
     Result:=List.Count;
  end;
end;


function GetValueFromList(NumParam: integer; List : TSTrings) : string;
//    (  )
Var
  Index : integer;
begin
  Result:='';
  if List<>NIL then begin
     Index:=NumParam-1;
     if (Index>=0) and (Index<List.Count) then begin
        Result:=Trim(List[Index]);
     end;
  end;
end;


function sp_LTrimUnChar(UnChar , Sx : string) : string;
//      ,    UnChar
Var
  YesExit : byte;
begin
  Result:=Sx;
  if (length(Sx)>0) and (length(UnChar)>0) then begin
     YesExit:=0;
     while YesExit<=0 do
      begin
        if POS(Result[1],UnChar)>0 then begin
           Delete(Result,1,1);
        end
        else begin
           YesExit:=2;
        end;
        if length(Result)<=0 then YesExit:=1;
     end;
  end;
end;

function sp_RTrimUnChar(UnChar , Sx : string) : string;
//      ,    UnChar
begin
  Result:=sp_InversionString(Sx);
  Result:=sp_LTrimUnChar(UnChar,Result);
  Result:=sp_InversionString(Result);
end;

function sp_AllTrimUnChar(UnChar , Sx : string) : string;
//        ,    UnChar
begin
  Result:=sp_LTrimUnChar(UnChar,Sx);
  Result:=sp_RTrimUnChar(UnChar,Result);
end;


function sp_InversionString(Sx : string) : string;
//  
Var
  i : integer;
begin
  Result:=Sx;
  if Length(Sx)>0 then begin
     Result:='';
     for i:=Length(Sx) downto 1 do
      begin
        Result:=Result+Sx[i];
     end;
  end;
end;


function sp_GetCountWordsInString(UnChar, InString : string) : integer;
//,  -   
//UnChar - ,  -
//InString -  ( )
var
  InWord : byte;
  i : integer;
begin
  Result:=0;
  InWord := 0;
  i:=0;
  while i<length(InString) do
   begin
    i:=i+1;
    if POS(InString[i],UnChar)<=0 then begin
       if InWord<=0 then Result:=Result+1; //   
       InWord:=1; //  
    end
    else begin
       InWord:=0; //  
    end;
  end;
end;


function sp_StrToInt(Sx : string; Def : integer) : integer;
//    
Var
  i,Res : integer;
begin
  Result:=Def;
  Sx:=Trim(Sx);
  Val(Sx,i,Res);
  if Res=0 then Result:=i;
end;


function sp_GetWordFromString(UnChar , InString : string; NumWord : integer) : string;
//,       
//UnChar - ,  -
//InString -  ( )
//NumWord -   (  1)
var
  InWord    : byte;
  NumWordCur, i : integer;
begin
  Result:='';
  if NumWord>0 then begin
    InWord := 0;
    NumWordCur:=0;
    i:=0;
    while i<length(InString) do
     begin
      i:=i+1;
      if POS(InString[i],UnChar)<=0 then begin
         if InWord<=0 then NumWordCur:=NumWordCur+1; //   
         InWord:=1; //  
         if NumWordCur=NumWord then begin // -  
            Result:=Result+InString[i]; //     
         end;
      end
      else begin
         InWord:=0; //  
         if NumWordCur=NumWord then begin //  -  
            i:=length(InString)+1; //,     
         end;
      end;
    end;
  end;
end;

function sp_CompareTwoString(Nr : byte;YesPartEQ,YesUpCase : Boolean;SFind,Sx : String) : Boolean;
//  
// SFind -  1
// Sx -  2
// Nr :  <=0 - , >0 - 
// YesPartEQ = TRUE -  
// YesUpCase = TRUE -    
begin
  Result:=FALSE;
  Sx:=Trim(Sx);
  SFind:=Trim(SFind);
  if (Length(SFind)>0) and (Length(Sx)>0) then begin
     if YesUpCase then begin
        UpperCase(Sx);
        UpperCase(SFind);
     end;
     if Nr<=0 then begin
        if POS(SFind,Sx)>0 then Result:=TRUE;
     end
     else begin
        if YesPartEQ then begin
           System.Delete(Sx,Length(SFind)+1,255);
        end;
        if SFind=Sx then Result:=TRUE;
     end;
  end;
end;


function sp_FindInTStrings(Nr : byte;YesPartEQ,YesUpCase : Boolean;SFind : string; List: TStrings) : integer;
//   
// SFind -  
// List - ,     SFind
// Nr :  <=0 - , >0 - 
// YesPartEQ = TRUE -  
// YesUpCase = TRUE -    
Var
  i : integer;
  Sx : string;
begin
  Result:=-1;
  if List<>NIL then begin
     if List.Count>0 then begin
        i:=-1;
        while i<(List.Count-1) do
         begin
          i:=i+1;
          Sx:=List[i];
          if sp_CompareTwoString(Nr,YesPartEQ,YesUpCase,SFind,Sx) then begin
             Result:=i;
             i:=(List.Count+1);
          end;
        end;
     end;
  end;
end;




//*******************************************************************
//*******************************************************************
//*******************************************************************
//   :
//http://politics.delphimaster.ru/cgi-bin/forum.pl?id=1188308666&n=0
function UTF8ToStr(Value: String): String;
{Convert string from UTF-8 format into ASCII}
var
 buffer: Pointer;
 BufLen: LongWord;
begin
 BufLen := Length(Value) + 4;
 GetMem(buffer, BufLen);
 FillChar(buffer^, BufLen, 0);
 MultiByteToWideChar(CP_UTF8, 0, @Value[1], BufLen - 4, buffer, BufLen);
 Result := WideCharToString(buffer);
 FreeMem(buffer, BufLen);
end;

function UTF8ToStrSmart(Value: String): String;
//  Value   UTF-8   ANSI
{Convert string from UTF-8 format mixed with standart ASCII symbols($00..$7f)}
var
 Digit: String;
 i: Word;
 HByte: Byte;
 Len: Byte;
begin
 Result := '';
 Len := 0;
 if Value = '' then Exit;
 for i := 1 to Length(Value) do
 begin
   if Len > 0 then
   begin
     Digit := Digit + Value[i];
     Dec(Len);
     if Len = 0 then
       Result := Result + UTF8ToStr(Digit);
   end else
   begin
     HByte := Ord(Value[i]);
     if HByte in [$00..$7f] then       //Standart ASCII chars
       Result := Result + Value[i]
     else begin
       //Get length of UTF-8 char
       if HByte and $FC = $FC then
         Len := 6
       else if HByte and $F8 = $F8 then
         Len := 5
       else if HByte and $F0 = $F0 then
         Len := 4
       else if HByte and $E0 = $E0 then
         Len := 3
       else if HByte and $C0 = $C0 then
         Len := 2
       else begin
         Result := Result + Value[i];
         Continue;
       end;
       Dec(Len);
       Digit := Value[i];
     end;
   end;
 end;
end;
//*******************************************************************
//*******************************************************************
//*******************************************************************


function UTF8_to_Str_List(List : TStrings): integer;
//  List   UTF-8   ANSI
Var
  i : integer;
  Sx : string;
begin
  Result:=0;
  if List<>NIL then begin
     if List.Count>0 then begin
        for i:=0 to (List.Count-1) do
         begin
           Sx:=List[i];
           Sx:=UTF8ToStrSmart(Sx);
           List[i]:=Sx;
        end;
     end;
  end;
end;




function sp_GetListOfWindowsNames(List : TStrings) : integer;
//      Windows
Const
  Sep = '_|=>';
Var
 hW : hWND;
 Exit99 : byte;
 Sx : string;
 CCC : array [0..255] of char;
begin
  Result:=0;
  if List<>NIL then begin
     List.Clear;
     hW:=GetTopWindow(0);
     if hW >32 then begin
        Exit99:=0;
        while Exit99<=0 do
         begin
          hW:=GetNextWindow(hW,GW_HWNDNEXT);
          if hW>32 then begin
             GetWindowText(hW,CCC,SizeOf(CCC));
             Sx:=StrPAS(CCC);
             Sx:=Trim(Sx);
             List.Add(Sx+Sep+IntToStr(hW));
          end
          else begin
             Exit99:=1;
          end;
        end;
     end;
     Result:=List.Count;
  end;
end;

function sp_GetHandleThisWindow(NameWin : string;Nr : byte;YesPartEQ,YesUpCase : Boolean) : hWND;
// Handle      (NameWin)
//NameWin -    
// Nr :  <=0 - , >0 - 
// YesPartEQ = TRUE -  
// YesUpCase = TRUE -    
Const
  Sep = '_|=>';
Var
  N,i : integer;
  Sx : string;
  List99 : TStrings;
begin
  Result:=0;
  List99:=TStringList.Create;
  TRY
    //       
    if sp_GetListOfWindowsNames(List99)>0 then begin
       //      ,
       //     NameWin
       i:=sp_FindInTStrings(Nr,YesPartEQ,YesUpCase,NameWin,List99);
       if i>=0 then begin
          Sx:=List99[i];
          N:=POS(Sep,Sx);
          if N>0 then begin
             //   -   
             System.Delete(Sx,1,N+Length(Sep)-1);
             Sx:=Trim(Sx);
             Result:=sp_StrToInt(Sx,0);
          end;
       end;
    end;
  FINALLY
    if List99<>NIL then begin
       List99.Clear;
       List99.Free;
       List99:=NIL;
    end;
  END;
end;



end.






